/***************************************************************************
 * BallotFactory.java                                                      *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *      
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * The BallotFactory class handles adding pages(Page.java) and              *
 * contest(Contest.java) to a ballot. This functionality is separate from   *
 * the Ballot class to simplify the logic of Ballot particularly,           *
 * eleminating the need for several temporary instance variables using in   *
 * building that ballot which are unused once voting begins.                *
 ****************************************************************************/


import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class BallotFactory{

    private Vector<Page> pages;
    private Vector<ReviewPage> reviewPages;
    private Vector<Contest> contests;
    private TreeMap<Integer,Integer> contestPageMapping;
    
    private Log log;
    
    private ReviewPage tempReview;
    private Integer pageCount;
    
    public BallotFactory(Log l){
	this.log = l;
	
	this.pages = new Vector<Page>();
	this.reviewPages = new Vector<ReviewPage>();
	this.contests = new Vector<Contest>();
	this.contestPageMapping = new TreeMap<Integer,Integer>();
	
	this.tempReview = null;
	this.pageCount = 0;
    }

    public Ballot getBallot(){
	return new Ballot(log,pages,reviewPages,contests,contestPageMapping);
    }
    
    // this should probably throw an exception for tempReview ==null
    // as that indicates a malformed ballot description.
    public void addContest(Contest c){
	
	if( this.pageCount > 3){
	    this.endSection();
	    this.startSection(c.getSectionText());
	}
	pageCount++;
	
	this.contestPageMapping.put(this.contestPageMapping.size() ,this.pages.size() - 1);
	
	this.pages.lastElement().addContest(c);
	this.tempReview.addContest(c);
	this.contests.add(c);
    }
    
    public void addNewPage(){
	this.pages.add( new Page( this.log) );
    }
    
    
    public void addTailPage(String text){
	this.reviewPages.add( new ReviewPage( this.log, text));
    }
    
    public void addTextPage(String text){
	this.pages.add( new Page( this.log, text));
    }
    
    public void startSection(String sectionText){
	this.tempReview = new ReviewPage(this.log, this.contests.size() , sectionText);
	this.pageCount = 0;
    }

    public void endSection(){
	this.reviewPages.add( tempReview);
    }
}
