/***************************************************************************
 * CandidateButton.java                                                    *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Reimplementation of a button class. The Buttons for candidates           *
 * Needed several lines of text, as well as highlighting. And access to the *
 * Candidate class.
 ****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class CandidateButton extends FakeButton{

    private final int ST_FONT_SIZE  = 16;
    private final int SST_FONT_SIZE = 16;

    protected JLabel subTitle;
    protected JLabel subSubTitle;
 
    protected Candidate candidate; 

    //public CandidateButton(){;}

    public CandidateButton(Log l, Candidate c){ 

	super(l);
	this.candidate = c;
	
	this.title       = this.initLabel(c.getTitle(),  T_FONT_SIZE);
	this.subTitle    = this.initLabel(c.getSubTitle(), ST_FONT_SIZE);
	this.subSubTitle = this.initLabel(c.getSubSubTitle(),SST_FONT_SIZE);

	this.setLayout(new GridLayout( c.numDefinedLabels() ,1) );

	
	this.add(this.title);
	if( !c.getSubTitle().equals("") )
	    this.add( this.subTitle);
	if( !c.getSubSubTitle().equals("") )
	    this.add(this.subSubTitle);
	
	this.setSelected( c.isSelected() );
    }
    
    public String toString(){
	return this.title.getText();
    }
      
    public void setTitle(String s){ this.setTitle(s, T_FONT_SIZE); }

    public void setTitle(String s, int fontSize){
	this.title.setText(s);
	this.title.setFont( new Font(title.getFont().getName(), title.getFont().getStyle(), fontSize));
    }
    
      public void setSelected(boolean b){
	if(b)
	    this.setBackground(Color.yellow);
	else
	    this.setBackground(Color.lightGray);
	
    }
    
    public String getText(){return this.title.getText();}
    public String getTitleText(){return this.title.getText();}
    public String getSubTitleText(){return this.subTitle.getText();}
    public String getSubSubTitleText(){return this.subSubTitle.getText();}

    public Integer getCandidateNum(){return candidate.getNum();}
    public boolean isWriteIn (){ return candidate.isWriteIn();}
    public boolean isSelected(){ return candidate.isSelected();}

    
    public boolean equals(CandidateButton b){
	return this.getTitleText().equals(b.getTitleText());
    }
}
