/***************************************************************************
 * ContestFactory.java                                                     *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Handle the construction of a given contest. As a ballot definition is    *
 * parsed the ContestFactory can be used to add candidates and set          *
 * properties.                                                              *
 ****************************************************************************/


import java.util.*;

public class ContestFactory {

    protected Log log;
    
    private Integer contestNum;
    private Integer maxSelections;
    private Integer numSelections;
    
    protected String sectionText;
    protected String raceText;
    protected String instructionText;
    
    protected boolean writeInEnabled;
    protected boolean twoPanelView;
    protected boolean hideSectionText;
    
    protected LinkedList<Candidate> candidates;
    
    
    public ContestFactory(){}
    
    public ContestFactory(Log l, Integer maxSel, String sText, String rText){
	this.log = l;

	this.writeInEnabled  = false;
	this.twoPanelView    = false;
	this.hideSectionText = false;

	this.sectionText     = sText;
	this.raceText        = rText;
	this.instructionText = "";
	
	this.contestNum    = 0;
	this.maxSelections = maxSel;
	this.numSelections = 0;
	this.candidates    = new LinkedList<Candidate>();
    }
    

    public void addCandidate(String p, String vp, String party){
        this.candidates.add( new Candidate( p,vp,party, this.candidates.size()));
    }
    
    public void setContestNum   (int c)    { this.contestNum = c;}
    public void setTwoColumnView(boolean t){ this.twoPanelView = t;}
    public void enableWriteIn   (boolean b){ this.writeInEnabled = b;}
    public void hideSectionText (boolean b){ this.hideSectionText = b;}
    
    public void setSectionText    (String s){ this.sectionText = s;}
    public void setRaceText       (String s){ this.raceText = s;}
    public void setInstructionText(String s){ this.instructionText = s;}

    public Contest getContest(){
	return new Contest(log,maxSelections, sectionText, raceText,
			   contestNum, writeInEnabled,twoPanelView,hideSectionText,
			   candidates);
    }
  
}
