/***************************************************************************
 * ContestView.java                                                        *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Display the information contained in a Contest class, and verify if the  *
 * modified mouse event interacted with any CandidateButtons.
 ****************************************************************************/


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ContestView extends JPanel{
    
    protected final int VGAP = 15;
    protected final int BUTTON_VGAP = 25;
    protected final int HGAP = 3;
    protected final int TEXT_GAP = 10;
    
    protected Log log;
    protected Ballot parent;
    
    protected JLabel section;
    protected JLabel race;
    protected JLabel instructions;
    
    protected LinkedList<CandidateButton> candidates;
    
    private JPanel main;
    private Contest contest;
    
    public ContestView(){}
    
    public ContestView(Log l,Contest c){
	this.log     = l;
	this.contest = c;
	
	this.setLayout(new BorderLayout( this.HGAP, this.TEXT_GAP));

	this.candidates = new LinkedList<CandidateButton>();
	for( Candidate cand : this.contest.getCandidates()){
	    this.candidates.add( new CandidateButton(log, cand));
	}


	this.setSectionText( c.getSectionText());
	this.setRaceText(    c.getRaceText() );
	this.instructions = new JLabel("");


	this.add(BorderLayout.NORTH,  this.getInstructionPanel());
	this.add(BorderLayout.CENTER, this.getCandidatePanel());
	this.setVisible(true);
  }
    
  
  public JLabel setLabelFont( JLabel l, int size){
    l.setFont( new Font(l.getFont().getName(), l.getFont().getStyle(), size ));
    return l;
  }
  
  public void setSectionText(String s){
    this.section = this.setLabelFont(new JLabel(s,SwingConstants.CENTER), 24);
  }
  
  public void setRaceText(String s){
    this.race = this.setLabelFont(new JLabel(s,SwingConstants.CENTER), 22);
  }
  
  public void setInstructionText(String s){
    this.instructions = this.setLabelFont(new JLabel(s,SwingConstants.CENTER), 20);
  }
  

    /**
     *
     * 
     */
    public EventMessage handleMouseClick(MouseEvent m){
      EventMessage em = EventMessage.newMissMSG();

      for( CandidateButton c : this.candidates){
	  if( c.containsPoint(m)){
	      if(!c.isWriteIn() || (c.isWriteIn() && contest.atMaxSelections()) ){
		  return EventMessage.newContestMSG(contest.getContestNum(),
						    c.getCandidateNum());
	      }
	      else if(c.isWriteIn() && !contest.atMaxSelections()){
		  return EventMessage.newStartWriteInMSG(contest.getContestNum(),
							 c.getCandidateNum(),
							 c.getText() );
	      }
	  }
      }
      return em;
  }
    

  protected JPanel getInstructionPanel(){
    int nrows = 0;
    GridLayout gl = new GridLayout ( 0 ,1, this.HGAP, this.VGAP);
    JPanel inst = new JPanel( gl );
  
    if( this.contest.getSectionText() != null && 
	!this.contest.hideSectionText ){
      //gl.setRows(++nrows);
      //inst.add(this.section);
    }
    if( this.contest.getRaceText() != null){
      gl.setRows(++nrows);
      inst.add(this.race);
    }
    if( this.contest.getInstructionText() != null &&
	!this.contest.getInstructionText().equals("") ){
      gl.setRows(++nrows);
      inst.add(this.instructions);
    }

    inst.setBorder(BorderFactory.createEmptyBorder(this.TEXT_GAP,0,0,0));    
    //inst.setMaximumSize( inst.getPreferredSize() );	
    return inst;
  }
  
  
  protected void addCandAndSpacer(JPanel p, Component c){
    p.add( c);
    p.add( Box.createRigidArea( new Dimension(CandidateButton.WIDTH, 0)  ));
  }


  protected JPanel getColumnPanel(java.util.List<CandidateButton> cand, int lb, int rb, boolean e){
    JPanel p = new JPanel();
    p.setBorder(BorderFactory.createEmptyBorder(0,lb,0,rb) );
    p.setLayout(new BoxLayout( p,  BoxLayout.Y_AXIS ));
    
    for( CandidateButton b : cand )
      this.addCandAndSpacer(p,b);

    //add an empty box for two column view with an odd # of candidates
    if( e )
      this.addCandAndSpacer(p, Box.createRigidArea( CandidateButton.getDefaultDimensions()));

    return p;
  }

  
  protected JPanel getCandidatePanel(){
    JPanel cPanel = new JPanel( );
    
    
    if( !this.contest.isTwoPanelView() ){
      cPanel.setLayout(new BoxLayout( cPanel, BoxLayout.Y_AXIS ));

      for( CandidateButton c : this.candidates)
	  this.addCandAndSpacer(cPanel, c);
    }    
    else{
      int size = candidates.size();
      cPanel.add( getColumnPanel( candidates.subList( 0, ((size)+1)/2) ,0,20, false));
      cPanel.add( getColumnPanel( candidates.subList( ((size +1) / 2) , size),20,0, (size%2)==1 ));
    }
    
    return cPanel;
  }

}
