/***************************************************************************
 * Controller.java                                                         *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Processes input from the user from both the screen(MouseListener) and    *
 * pollworker interface(KeyListener). Make changes to the system state      *
 * stored in Model and Ballot, and initiate changes to the View.            *
 ****************************************************************************/


import java.util.Timer;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;



public class Controller{

    private Model model;
    private View view;
    private Log log;
    private DataEntry dataEntry;
    
    private boolean demo;
    private boolean reset;

    private long downTime;
    private boolean released;
    private boolean waitForTimer;

    private Timer timer;
    private MouseDelay task;

  
    public Controller(Model m, View v, Log l){
    this.model = m;
    this.view = v;
    this.log = l;
    this.dataEntry = new DataEntry(l);

    this.demo  = false; 
    this.reset = false;

    this.downTime = 0;
    this.released = false;
    this.waitForTimer = false;

    this.downTime = 0;
    this.timer = new Timer();
    this.task = null;

    this.view.addKeyListener   (new Kl(view));
    this.view.addMouseListener (new Ml(view, this));
  }
  



    public  long    downTime(){      return this.downTime;}
    public  synchronized boolean touchReleased(){ return this.released;}
    public  synchronized boolean getWaitForTimer(){ return this.waitForTimer;};
    private synchronized void setReleased(boolean t){ this.released = t;}
    public  synchronized void setWaitForTimer(boolean w){ this.waitForTimer = w;}

    private class Ml implements MouseListener{
	private Controller parent;

        public Ml(View v, Controller c){ parent = c;}
        public void mouseClicked(MouseEvent e){;}
        public void mouseEntered(MouseEvent e){;}
        public void mouseExited(MouseEvent e){;}

        public void mousePressed(MouseEvent e){
	    downTime = System.nanoTime();
	    log.startEvent();
	    log.log("Touch","" );

            if( model.isMiscalibrated() )
                e = model.miscalibrate(e);

            if( !model.isInsensitive() )
                handleMouseClick(e);
	    else if(  model.isInsensitive() && !getWaitForTimer() ){
		task = new MouseDelay(parent, model.getDelayTime(),e);
		setWaitForTimer(true);
		timer.schedule(task, 0);
	    }

	    setReleased(false);
        }

        public void mouseReleased(MouseEvent e){
	    setReleased(true);

	    if( model.isInsensitive() && getWaitForTimer()){
		try{
		    task.cancel();
		    setWaitForTimer(false);
		}
		catch(IllegalStateException ise){System.out.println(ise);}

		log.log("DelayInvalid", "");
	    }

	    log.log("Release","");
	    while( getWaitForTimer() ){ ; }
	    log.endEvent();
        }
    }

    public synchronized void handleMouseClick(MouseEvent e){
	EventMessage event = view.checkMouseEvent(e);
	Ballot ballot = model.getBallot();


        if( event.isNavNext() )
            model.getBallot().navigateNext();


        else if( event.isNavPrev() )
            model.getBallot().navigatePrev();

        else if( event.isNavSummary() )
            model.getBallot().navigateToSummary();

	else if( event.isContest())
	    model.getBallot().updateContest(event.getData1(),event.getData2());
	
	else if( event.isStartWriteIn()){
	    this.log.log("WriteInDisplayed","true");
	    model.getBallot().setCurrentWriteIn(event.getData1(),event.getData2());
	}

	else if( event.isReview())
	    model.getBallot().reviewContest( event.getData1() );

	else if( event.isWriteIn()){
            if (event.getText().equals("Accept")){
		ballot.acceptWriteIn();
                ballot.setWriteInMode(false);
		ballot.setWriteInText("");
		this.log.log("WriteInDisplayed","false");
	    }
	    else if (event.getText().equals("Cancel")){
                model.getBallot().setWriteInMode(false);
		model.getBallot().setWriteInText("");
		this.log.log("WriteInDisplayed","false");
	    }
	    else{ 
		ballot.setWriteInText( event.getText());
	    }
	}

	
	if( model.getBallot().hasUpdates() ){
	    view.getModelUpdates();
	    model.getBallot().updatesComplete();
	}

	if (model.getBallot().atLastPage())
	    log.log("BallotCast","true");

	this.log.log("Hit", Boolean.toString(!event.isMiss()));
    }

    private class Kl implements KeyListener{
        public Kl(View v){}
        public void keyTyped(KeyEvent e) {}
        public void keyPressed(KeyEvent e){}
        public void keyReleased(KeyEvent e) {
	    int keyCode = e.getKeyCode();
	    if(keyCode == KeyEvent.VK_ESCAPE){
		log.close();
		System.exit(0);
	    }
	    else if(keyCode == KeyEvent.VK_MULTIPLY)
		beginStandardSessionReset();

	    else if(keyCode == KeyEvent.VK_ADD)
		beginDemoSessionReset();

	    else if( demo && isValidDemoKeyCode(keyCode) )
		finishDemoSessionReset(e);

	    //Read entry on the demographic keypad 1 value at a time
	    else if( reset && !demo ){
		reset = !dataEntry.getVoterDemographics(e);
		if(!reset)
		    finishStandardSessionReset();
	    }
        }

	/**
	 * These are all helpers for Kl.keyReleased(e);
	 */
	private void beginStandardSessionReset(){
	    view.clearScreen();
	    model.selectMode();
	    model.initNewSession();
	    dataEntry.enterNewVoter( model.getMode(),
				     model.getModeOptions(),
				     model.getNumVoters());
	    reset = true;
	}

    
	private void beginDemoSessionReset(){
	    view.clearScreen();
	    demo = true;
	}
	
	private void finishStandardSessionReset(){
	    model.demographicsEntered();
	    view.getModelUpdates();
	}
	
	private void finishDemoSessionReset(KeyEvent e){
	    model.selectDemoMode(e);
	    model.initNewSession();
	    view.getModelUpdates(); 
	    demo = false;
	}
	
	private boolean isValidDemoKeyCode(int keyCode){
	    return keyCode >= 96 && keyCode <= 101;
	}	
    }
}

