/***************************************************************************
 * DataEntry.java                                                          *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * DataEntry processes the input of the "pollworker interface" and prints   *
 * that information to the Event Log, before the events for a new voter.    *
 *                                                                          *
 * DataEntry also contains the following private class.                     *
 *  - InputEvent: Helper for handling pollworker input                      *  
 ****************************************************************************/



import java.util.*;
import java.io.*;
import java.awt.event.*;

public class DataEntry{

    String rq[][] =
    { {"Age:", "18-30", "31-45", "46-60", "61-75", "75+"},
      {"Gender", "Male", "Female"},
      {"Handedness", "Right", "Left"},
      {"EducationLevel", "No Diploma", "HS Diploma" ,"BS/BA", "Grad"},
      {"Race-Eth", "white" ,"black", "hispanic", "native", "asian", "multi", "other"},
      {"PastVotingRate", "Rarely/Never", "Pres", "ConPres", "all"},
      {"DRE", "yes" ,"no"},
      {"OpticalScan", "yes", "no"},
      {"Lever",  "yes", "no"},
      {"PunchCard", "yes", "no"},
      {"MonitorVision", "yes", "no"},
      {"HearAudio", "yes", "no"},
      {"UseMouse", "yes",  "no"},
      {"ComputerUse", "Rarely/Never" ," Less 5" ,"5-10" ,"10-20" ,"20-40" ,"40+" },
      {"WebBrowser",  "Rarely/Never" ," Less 5" ,"5-10" ,"10-20" ,"20-40" ,"40+" }};
    
    
    private StringBuffer sb;
    private PrintStream display;
    private Log log;
    
    private Integer pos;
    InputRequest r[];
    
    
    public DataEntry(Log l){
	this.sb      = new StringBuffer();
	this.log     = l;
	this.pos     = 0;
	this.r       = this.createInputRequests( rq );

	try{
	    this.display = new PrintStream( "poll.txt");}
	catch(FileNotFoundException fnfe ){
	    System.err.println("Failed to open the pollworker display file");
	    this.display = System.out;
	}
	
    }
    
    public void enterNewVoter(String mode, String options, Integer numVoters){

	this.pos = 0;
	if(numVoters > 1)
	    this.log.println("</Voter>");
	
	this.log.print("<Voter ");
	this.log.print("mode=\"" + mode + "\"");
	this.log.print(" options=\"" + options + "\"");
	this.log.println(" number=\"" +  numVoters.toString() + "\">");
    }
    
    
    public InputRequest[] createInputRequests(String lst[][] ){
	InputRequest r[] = new InputRequest[ lst.length ];
	
	for(int i = 0; i < lst.length ; i++)
	    r[i] = new InputRequest( lst[i] );
	
	return r;
    }
    
    
    
    
    
    public boolean getVoterDemographics(KeyEvent e){
	if( this.pos < r.length){
	    try{
		char c = e.getKeyChar();
		if( Character.isDigit(c) ){
		    // Set the value for the current InputRequest,
		    // require reentry on invalid input
		    if( r[this.pos].setResponse( Character.digit(c,10) )){
			this.display.println( Character.toString(c) );
			this.pos++;
		    }
		    else
			this.display.println("Value out of range: " + this.pos.toString()); 
		}
		//Re-enter the previous InputRequest
		else if( (c == '-') && this.pos > 0)
		    this.pos--;
		
		if( this.pos ==  r.length)
		    for( int i = 0 ; i < r.length ; i++)
			this.log.println(r[i].toString());
		
	    }
	    catch(Exception ex){System.err.println("error reading demographics " +ex);}
	}
	
	return this.pos ==  r.length;
    }
    
    

    
    private class InputRequest{
	private String   name;
	private Integer  max;
	private String[] options;
	private Integer  response;

	/**
	 * The input options are almost identical to the args
	 * passed to the constructor. The constructor takes a list
	 * of the form {name, "arg1", ... ,"argn"}. arg1-argn
	 * are the options, representing inputs 1-n on the kaypad.
	 * name, is replaced with the string "none", which will 
	 * be printed to the log in the event of a skipped answer.
	 */
    public InputRequest(String args[]){
	try{
	    this.max = args.length;
	    this.options = args;
	    this.response = 0;
	    
	    if( this.max < 1){
		System.err.println("Input options cannot be empty. Exiting");
		System.exit(1);
	}
	    
	    this.name = this.options[0];
	    this.options[0] = "none";
	}
	catch(Exception IndexOutOfBounds){
	    System.err.printf("array bounds less 0 %b\n", args.length < 0);
	    System.exit(1);
	}
	
    }
	public boolean setResponse( int i){
	    if( i <= this.max)
		this.response = i;
	    return i <= this.max;
	}
	
	public String getResponse(){
	    return this.options[ this.response ];
	}
	
	public String getInstructions(){
	    return this.name;
	}
	
	public String toString(){
	    return this.toXmlString();
	}
	
	public String toTextString(){
	    return name + " " + this.getResponse();
	}
	
	public String toXmlString(){
	    return "<" + name + ">" + this.getResponse() + "</" + name + ">";
	}

    }
    
}
