/***************************************************************************
 * EventMessage.java                                                        *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2010 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * September 2010                                                           *
 *                                                                          *
 * Provide data about what kind of button was pressed(or missed) on the     *
 * screen as well as relevent data from that button.                         *
 ****************************************************************************/

public class EventMessage{
    
    public enum EventType {NAV_N,NAV_P,NAV_S, CONTEST, START_WRITEIN,WRITEIN, REVIEW,MISS,NONE}

    private EventType type;
    private String    sdata;
    private Integer   ndata1;
    private Integer   ndata2;



    public EventMessage(){
	init(EventType.NONE,0,0,"");
    }

    public EventMessage(EventType t){
        init(t,0,0,"");
    }

    public EventMessage(EventType t, Integer nd){
        init(t,nd,0,"");
    }

    public EventMessage(EventType t, Integer nd1, int nd2){
        init(t,nd1,nd2,"");
    }

    public EventMessage(EventType t, Integer nd1, int nd2, String sd){
        init(t,nd1,nd2,sd);
    }

    public EventMessage(EventType t, String sd){
        init(t,0,0,sd);
    }

    public void init(EventType t, Integer nd1, int nd2, String sd){
	type  = t;
        ndata1 = nd1;
        ndata2 = nd2;
        sdata = sd;
    }

    
    public static EventMessage newNextMSG   (){ return new EventMessage(EventType.NAV_N);}
    public static EventMessage newPrevMSG   (){ return new EventMessage(EventType.NAV_P);}
    public static EventMessage newSummaryMSG(){ return new EventMessage(EventType.NAV_S);}
    public static EventMessage newMissMSG   (){ return new EventMessage(EventType.MISS);}
    public static EventMessage newWriteInMSG(){ return new EventMessage(EventType.WRITEIN);}

    public static EventMessage newReviewMessage(int contest){
	return new EventMessage(EventType.REVIEW,contest);
    }

    public static EventMessage newWriteInMSG(String s){ 
	return new EventMessage(EventType.WRITEIN,s);
    }

    public static EventMessage newContestMSG(int contest, int option){ 
	return new EventMessage(EventType.CONTEST,contest, option);
    }

    public static EventMessage newStartWriteInMSG(int contest, int option, String name){ 
	return new EventMessage(EventType.START_WRITEIN,contest, option,name); 
    }


    public boolean isNavNext     (){ return type == EventType.NAV_N;}
    public boolean isNavPrev     (){ return type == EventType.NAV_P;}
    public boolean isNavSummary  (){ return type == EventType.NAV_S;}
    public boolean isMiss        (){ return type == EventType.MISS;}
    public boolean isContest     (){ return type == EventType.CONTEST;}
    public boolean isWriteIn     (){ return type == EventType.WRITEIN;}
    public boolean isReview      (){ return type == EventType.REVIEW;}
    public boolean isStartWriteIn(){ return type == EventType.START_WRITEIN;}
    public boolean hasText       (){ return sdata.length() > 0;}


    public int getData1  (){ return ndata1;}
    public int getData2  (){ return ndata2;}
    public String getText(){ return sdata;}
    

    public String toString(){
	StringBuffer sb = new StringBuffer();
	return sb.toString();
    }
}





