/***************************************************************************
 * FakeButton.java                                                         *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Implementation of a button class. We needed to define our own button     *
 * class to allow simulation of miscalibration.                             *
 ****************************************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class FakeButton extends JPanel{
    
    protected final int T_FONT_SIZE   = 18;
    
    public final static int WIDTH = 350;
    public final static int HEIGHT = 90;
    
    protected JLabel title;  
    protected Log    log;
    
    
    
    public FakeButton(){;}
    public FakeButton(Log l){           this.init(l, "");}
    public FakeButton(Log l, String t){ this.init(l,t);}

    private void init(Log l, String t){
	Dimension size = new Dimension(this.WIDTH, this.HEIGHT);
	
	this.log  = l;
	this.title = initLabel(t,T_FONT_SIZE);
	if( !t.equals("") )
	    this.add(title);
	
	this.setSize(size);
	this.setPreferredSize(size);
	this.setMaximumSize(size);
	this.setMinimumSize(size);
	
	this.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1) );
	
	this.setBackground(Color.lightGray);
	this.setVisible( true );
    }
    
    
    protected JLabel initLabel(String s, Integer fontSize){
	JLabel l = new JLabel(s, SwingConstants.CENTER);
	l.setFont( new Font(l.getFont().getName(), l.getFont().getStyle(), fontSize));
	
	l.setVerticalAlignment(SwingConstants.CENTER);
	return l;
    }
    
    
    
    public void setText(String s){ this.setText(s, T_FONT_SIZE); }
    
    public void setText(String s, int fontSize){
        this.title.setText(s);
        this.title.setFont( new Font(title.getFont().getName(), title.getFont().getStyle(), fontSize));
    }
    
    
    public Log    getLog(){return this.log;}
    
    public static Dimension getDefaultDimensions(){
	return new Dimension(FakeButton.WIDTH, FakeButton.HEIGHT);
    }
    
    public String toString(){
        return this.title.getText();
    }

    protected boolean containsPoint(MouseEvent m){
	boolean contained = false;
	
	try{
	    Point loc = this.getLocationOnScreen();
	    contained = (( m.getX() >= (int)loc.getX() ) && 
			 ( m.getY() >= (int)loc.getY() ) && 
			 ( m.getX() <= (int)loc.getX() + this.getWidth()) && 
			 ( m.getY() <= (int)loc.getY() + this.getHeight()));
	    
	    if(contained)
		this.log.log("RelCoord", String.format("%d:%d", ( m.getX()-(int)loc.getX()),
						       (m.getY()- (int)loc.getY())));
	    
	}
	catch(IllegalStateException e){System.err.println("Component Not visible " ); }
	
	return contained;
    }    
}
