/***************************************************************************
 * NavigationPanel.java
 ****************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * August 2010                                                              *
 *                                                                          *
 * This is a JPanel that contains three buttons to control Navigation:      *
 * Next, Previous and Return to Summary. Based on where the current page    *
 * is located in the ballot, not all buttons should be displayed.           *
 * Any time the user is viewing a contest page from the summary screen      *
 * previous and next buttons will be disabled. Return to Summary will be    *
 * only option.                                                             * 
 ****************************************************************************/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class NavigationPanel extends JPanel{
  
    protected final int NAV_BORDER_TOP = 20;
    protected final int NAV_BORDER_LEFT = 30;
    protected final int NAV_BORDER_BOTTOM = 30;
    protected final int NAV_BORDER_RIGHT = 30;
    
    protected String NEXT_TEXT = "Next";
    protected String PREV_TEXT = "Previous";
    
    protected FakeButton prevButton;
    protected FakeButton nextButton;
    protected FakeButton summaryButton;
    
    protected Log log;
    
    public NavigationPanel (){}
    
    public NavigationPanel(Log l, boolean prev, boolean summary, boolean finalNav, boolean finalPage){
	this.log = l;

	this.setLayout( new BorderLayout());
	
	this.prevButton = new FakeButton( log, PREV_TEXT);
	this.nextButton = new FakeButton( log, NEXT_TEXT);
	
	this.summaryButton = new FakeButton(log, "Return to Summary");
	this.summaryButton.setVisible(false);
	
	this.add(BorderLayout.WEST,  this.prevButton);
	this.add(BorderLayout.CENTER,this.summaryButton  );
	this.add(BorderLayout.EAST,  this.nextButton);
	
	this.setBorder(BorderFactory.createEmptyBorder(NAV_BORDER_TOP, NAV_BORDER_LEFT,
						       NAV_BORDER_BOTTOM, NAV_BORDER_RIGHT) );
	
        this.nextButton.setVisible( !summary && !finalPage);
        this.prevButton.setVisible( (prev && ! summary) && !finalPage);
        this.summaryButton.setVisible( summary);
	
	if(finalNav)
	    setFinalNavText();

	this.setVisible(true);
    }
    
    public boolean isNavigateNext(MouseEvent m){
	return this.nextButton.isShowing() && this.nextButton.containsPoint( m );}
    public boolean isNavigatePrev(MouseEvent m){
	return this.prevButton.isShowing() && this.prevButton.containsPoint( m); }
    public boolean isNavigateSummary(MouseEvent m){
	return this.summaryButton.isShowing() && this.summaryButton.containsPoint( m );}
    
    
    public void setFinalNavText(){
	this.nextButton.setText("Cast Ballot",22);
	this.prevButton.setText("Go Back",22);
	this.setLayout(new FlowLayout() );
	this.setBorder(BorderFactory.createEmptyBorder(0,0, NAV_BORDER_BOTTOM + FakeButton.HEIGHT * 5,0) );
    }
}
