/***************************************************************************
 * PageView.java                                                           *
****************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Display the data contained in a Page object.
 ****************************************************************************/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.LinkedList;

public class PageView extends JPanel{
  
    protected Log log;
    protected Ballot parent;
    
    protected JPanel contestPanel;
    protected NavigationPanel navPanel;
    
    protected Page page;
    protected LinkedList<ContestView> contests;

  public PageView (){}

    public PageView(Log l, Page p, boolean atReview){
	this.log = l;
	this.page = p;
	
	this.setLayout( new BorderLayout());
	
	this.contests = new LinkedList<ContestView>();

	
	this.navPanel = new NavigationPanel(l,p.prevEnabled(), atReview, p.isFinalNav(), p.isFinalPage());
	this.contestPanel = new JPanel();
	this.contestPanel.setLayout( new BoxLayout(this.contestPanel, BoxLayout.Y_AXIS) );
	
	for(Contest c : p.getContests() ){
	    ContestView cv = new ContestView(l,c);
	    contests.add(cv);
	    contestPanel.add( cv);
	}

	this.add(BorderLayout.NORTH,  this.contestPanel);
	this.add(BorderLayout.SOUTH,  this.navPanel );
	
	this.contestPanel.setVisible(true);
	this.navPanel.setVisible(true);
	this.setVisible(true);
	
    }
    
    public boolean isNavigateNext(MouseEvent m){    return navPanel.isNavigateNext( m );}
    public boolean isNavigatePrev(MouseEvent m){    return navPanel.isNavigatePrev( m ); }
    public boolean isNavigateSummary(MouseEvent m){ return navPanel.isNavigateSummary( m );}

    public EventMessage handleMouseClick(MouseEvent m){
	EventMessage em = EventMessage.newMissMSG();

	for( ContestView c : this.contests ){
	    em = c.handleMouseClick(m);
	    if (!em.isMiss() )
		return em;
	}

	return em;
    }
    
     
}
