/***************************************************************************
 * ReviewPage.java
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************                                                               
 * Paul Cotton                                                              *                                                         
 * The University of Iowa                                                   *                                                         
 * Department of Computer Science                                           *                                                         
 * April 2009                                                               *                                                         
 *                                                                          *                                                         
 * This is an extension of the Page class, used only for review races.      * 
 * The review process is a special case for navigation, since it allows     *
 * non-linear navigation back to contests. There are also some functions    *
 * in here which create the 'dishonest summary screen' experiment in the    *
 * presidential race.                                                       *
 ****************************************************************************/




import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class ReviewPage extends Page{

    private final Integer MAX_MODIFICATIONS = 2;
    
    private Integer contestOffset;
    private LinkedList<Contest> contests;
    
    private Integer dishonestContest;
    private boolean dishonestyCorrected;
    private String  raceLabel;
    
    private Log log;

    public ReviewPage(Log lg, Integer offset, String l){
	super(lg);
	this.contestOffset = offset;
	
	this.dishonestContest = 0;
	this.dishonestyCorrected = true;
	
	this.contests   = new LinkedList<Contest>();
	this.raceLabel = "Reviewing " + l;
    }

    public ReviewPage(Log l, String t){
	super(l,t);

        this.contests   = new LinkedList<Contest>();
        this.dishonestContest = 0;
        this.dishonestyCorrected = true;
	this.contestOffset =0;
	this.raceLabel = " ";
    }

    public void addContest( Contest c){
	this.contests.add(c);
    }


    /**
     * initDishonesty()
     *
     * @param boolean isDishonest
     *
     * The behavior of initdishonesty may appear strange at first. In the
     * case of an honest system this.dishonestyCorrected is set to true.
     * This prevents changes from being made in the case of honesty. It should
     * only be set to false at the beginning of a dishonest session.
     **/
  public void initDishonesty(boolean isDishonest){
    this.dishonestContest = 0;
    this.dishonestyCorrected = !isDishonest;
  }

  
  
    public void setDishonest(){
	if(  !dishonestyCorrected){
	    this.contests.getFirst().setDishonestCandidate();
	    this.dishonestyCorrected = true;
	}
    }
    

    public LinkedList<Contest> getContests(){return contests;}
    public String  getRaceLabel    (){ return this.raceLabel;}
    public Integer getContestOffset(){ return this.contestOffset;}
}
