/***************************************************************************
 * ReviewPageView.java
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************                                                               
 * Paul Cotton                                                              *                                                         
 * The University of Iowa                                                   *                                                         
 * Department of Computer Science                                           *                                                         
 * April 2009                                                               *                                                         
 *                                                                          *                                                         
 * Display the data contained in a ReviewPage.                              *
 ****************************************************************************/




import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class ReviewPageView extends PageView{

    private final Integer VGAP = 30;
    private final Integer HGAP = 10;
    private final Dimension spacer = new Dimension(CandidateButton.WIDTH, this.VGAP);

    private ReviewPage reviewPage;
    private LinkedList<CandidateButton> buttons;
    
    public ReviewPageView(Log l, ReviewPage r){
	
	this.log = l;
	this.reviewPage = r;
	this.setLayout( new BorderLayout(this.HGAP, this.VGAP));
	
	this.navPanel = new NavigationPanel(l,r.prevEnabled(), false, r.isFinalNav(), r.isFinalPage());
	this.buttons = new LinkedList<CandidateButton>();

	this.contestPanel = new JPanel();
	this.contestPanel.setLayout( new BoxLayout(this.contestPanel, BoxLayout.Y_AXIS) );
	this.initButtons();

	JLabel label = new JLabel(reviewPage.getRaceLabel() );
	label.setFont( new Font(label.getFont().getName(), label.getFont().getStyle(), 24));
	label.setHorizontalAlignment(SwingConstants.CENTER);
	label.setBorder(BorderFactory.createEmptyBorder(this.HGAP * 2,0,0,0));
	
	this.add(BorderLayout.NORTH,  label);
	this.add(BorderLayout.CENTER, this.contestPanel);
	this.add(BorderLayout.SOUTH,  this.navPanel);
	
	this.setVisible(true);

    }

    
    private void initButtons(){
        for( Contest c : this.reviewPage.getContests() ){
            int i = c.getMaxSelections();
            for( Candidate b : c.getCandidates()){
                if( b.isSelected() ){
                    this.addButton( new CandidateButton(log, b),c);
                    i--;
                }
            }

            while( i > 0){
                this.addButton( new CandidateButton(log, new Candidate("No Selection Made","","",1)),c);
                i--;
            }
        }

    }

  
    private void addButton( CandidateButton b, Contest c){
	JLabel label = new JLabel( c.getRaceText() );
	JPanel p1 = new JPanel();
	JPanel p2 = new JPanel();
	label.setFont( new Font(label.getFont().getName(), label.getFont().getStyle(), 22));
	p1.setLayout( new BoxLayout(p1, BoxLayout.X_AXIS) );
	p2.setLayout( new BoxLayout(p2, BoxLayout.X_AXIS) );
	
	p1.add( label); 
	p2.add( b);
	
	this.contestPanel.add(p1);
	this.contestPanel.add(p2);
	
	this.contestPanel.add( Box.createRigidArea( this.spacer ));
	this.buttons.add( b );
    }
    
    
    public EventMessage handleMouseClick(MouseEvent m){
	int i = this.reviewPage.getContestOffset();
	int j = 0;
	EventMessage e = EventMessage.newMissMSG();
	
	for( Contest c : this.reviewPage.getContests()){
	    int maxSel = c.getMaxSelections();
	    while( j < buttons.size() && maxSel > 0){
		CandidateButton b = this.buttons.get(j);
		if(b.containsPoint( m )){
		    return EventMessage.newReviewMessage(i);
		}
		maxSel--;
		j++;
	    }
	    i++;
	}
	
	
	return e;
    }
}
