/***************************************************************************
 * View.java                                                               *
 ****************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * This handles screen initialization, displays the PageView associated     *
 * with the current page in the ballot, and checks whether modified mouse   *
 * events occured over and FakeButtons. 
 *****************************************************************************/

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.awt.image.MemoryImageSource; 
import java.util.Timer;


public class View extends JFrame{
  
    private Model model;
    private Log log;

    private PageView currentPage;
    
    public View(Model m, Log l, boolean pointer){
	this.model = m;
	this.log = l;
	
	this.currentPage = new PageView();
	
	this.setDefaultCloseOperation(EXIT_ON_CLOSE);
	this.setUndecorated( true );
	this.setVisible(true);
	
	int[] pixels = new int[16 * 16];
	Image image = Toolkit.getDefaultToolkit().createImage(
							      new MemoryImageSource(16, 16, pixels, 0, 16));
	Cursor transparentCursor =
	    Toolkit.getDefaultToolkit().createCustomCursor
	    (image, new Point(0, 0), "invisibleCursor");
	
	if( !pointer)
	    setCursor(transparentCursor);
	this.enterFullScreen();
    }
    
    public void enterFullScreen(){
	GraphicsEnvironment graphicsEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
	GraphicsDevice graphicsDev = graphicsEnv.getDefaultScreenDevice();
	graphicsDev.setFullScreenWindow(this);
	
	this.validate();    
    }
    

    public void getModelUpdates(){

	this.remove(this.currentPage);
	
        if(model.getBallot().isWriteInMode())
            this.currentPage = new WriteInPage(log, model.getBallot().getWriteInText() );

	else if(model.getBallot().getCurrentPage().hasText())
	    this.currentPage = new TextPageView(log,
					    model.getBallot().getCurrentPage());

	else if(!model.getBallot().atReviewPage())
	    this.currentPage = new PageView (log, 
					     model.getBallot().getCurrentPage(),
					     model.getBallot().inReviewMode());

	else
	    this.currentPage = new ReviewPageView (log, model.getBallot().getCurrentReviewPage());

	currentPage.setVisible(true);

	this.add( currentPage );
	
	this.validate();
        this.repaint(); 
	this.log.log("Feedback", "");;
    }

    public void clearScreen(){
	this.remove(this.currentPage);
        this.validate();
        this.repaint();
    }

    public synchronized EventMessage checkMouseEvent(MouseEvent m){
	EventMessage event = null;

	if( this.currentPage.isNavigateNext(m) )
	    event = EventMessage.newNextMSG();
	else if( this.currentPage.isNavigatePrev(m))
	    event = EventMessage.newPrevMSG();
	else if( this.currentPage.isNavigateSummary(m) )
	    event = EventMessage.newSummaryMSG();
	else{
	    event = this.currentPage.handleMouseClick(m);

	}

	return event;
    }

}
