/***************************************************************************
 * WriteInPage.java                                                        *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************                                                    
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * This page allows for write-in candidates. It provides an on screen       *
 * keyboard, and accept/cancel buttons to return the voter to contest       * 
 * pages.                                                                   *
 ****************************************************************************/

import java.awt.*;
import java.awt.event.MouseEvent;
import javax.swing.*;
import java.util.LinkedList;

public class WriteInPage extends PageView{

    private final int LETTER_WIDTH  = 80;
    private final int LETTER_HEIGHT = 80;

    protected final int NAV_BORDER_TOP = 20;
    protected final int NAV_BORDER_LEFT = 30;
    protected final int NAV_BORDER_BOTTOM = 30;
    protected final int NAV_BORDER_RIGHT = 30;
    
    private final int SPACE = 11;
    private final int NROWS = 3;
    private final int NCOLS = 10;
    private final int ASCII_A = 65;
    private final int ASCII_Z = 91;
    private final int MAX_WRITE_IN_LENGTH = 30;
    
    private boolean accepted;
    private Integer position;
    private StringBuffer  entry;
    
    private FakeButton accept;
    private FakeButton cancel;
    
    private FakeButton clear;
    private FakeButton backspace;
    private LinkedList<FakeButton> charKeys;
    
    private JLabel display;
    private JPanel displayFrame;
    
    private FakeButton space;
    
    public WriteInPage(Log l, String s){
	this.log = l;
	this.accepted = false;
	this.entry = new StringBuffer(s);
	
	this.space     = new FakeButton( l, "Space");
	this.clear     = new FakeButton( l, "Clear");
	this.backspace = new FakeButton( l, "Backspace"); 
	this.charKeys = new LinkedList<FakeButton>();
	
	this.display = new JLabel( s + " ");
	this.displayFrame = new JPanel();
	
	this.setLayout( new BorderLayout());

	JLabel inst = new JLabel("Enter the name using the On-Screen keyboard.",
				 SwingConstants.CENTER);
	inst.setFont( new Font(inst.getFont().getName(), inst.getFont().getStyle(), 20 ));
	
	
	this.add(BorderLayout.NORTH,  inst);
	this.add(BorderLayout.CENTER, this.getKeyboardPanel());
	this.add(BorderLayout.SOUTH,  this.getNavigationPanel());
	this.setVisible(true);
    }
    
    
    private JPanel getNavigationPanel(){
	JPanel navigation = new JPanel( new BorderLayout());
	
	this.cancel = new FakeButton( log, "Cancel");
	this.accept = new FakeButton( log, "Accept");
	navigation.add(BorderLayout.WEST, this.cancel);
	navigation.add(BorderLayout.EAST, this.accept);
	navigation.setBorder(BorderFactory.createEmptyBorder(NAV_BORDER_TOP,
							     NAV_BORDER_LEFT,
							     NAV_BORDER_BOTTOM * 2,
							     NAV_BORDER_RIGHT ) );
	return navigation;
    }
    
    private JPanel initKeys(){
	JPanel keys = new JPanel( new GridLayout(NROWS, NCOLS, SPACE, SPACE) );
	String[] top = {"Q","W","E","R","T","Y","U","I","O","P",
			"A","S","D","F","G","H","J","K","L","-",
			"Z","X","C","V","B","N","M",",",".","~"};
	
	for(int i = 0 ; i < top.length ; i++){
	    FakeButton temp = new FakeButton( log, top[i] );
	    keys.add( temp );
	    this.charKeys.add( temp );
	}
	
	keys.setMaximumSize(new Dimension( NCOLS * (LETTER_WIDTH  + SPACE),
					   NROWS * (LETTER_HEIGHT + SPACE)));
	
	keys.setBorder(BorderFactory.createEmptyBorder(0,1,0,1 ) );
	
	return keys;
    }
    
    
    
    
    private JPanel getKeyboardPanel(){
	Dimension spacer = new Dimension(SPACE, SPACE);
	
	JPanel keyboard = new JPanel();
	keyboard.setLayout(new BoxLayout( keyboard, BoxLayout.Y_AXIS ));
	
	JPanel delete = new JPanel();
	JPanel keys = new JPanel( new GridLayout(NROWS, NCOLS, SPACE, SPACE) );
	
	display.setFont( new Font(display.getFont().getName(), display.getFont().getStyle(), 24));
	displayFrame.add(this.display);
	displayFrame.setMaximumSize(new Dimension( FakeButton.WIDTH*2, FakeButton.HEIGHT ));
	displayFrame.setMinimumSize(new Dimension( FakeButton.WIDTH*2, FakeButton.HEIGHT ));
	displayFrame.setBackground(Color.yellow);
	
	delete.add(this.clear);
	delete.add(this.backspace);
	
	keys = this.initKeys();
	
	keyboard.add(displayFrame);
	keyboard.add(delete);
	keyboard.add( Box.createRigidArea( spacer ) );
	keyboard.add(keys);
	keyboard.add( Box.createRigidArea( spacer ) );
	keyboard.add(this.space);
	keyboard.add( Box.createVerticalGlue());
	
	return keyboard;
    }
    

    public String getEntry(){ return this.entry.toString();}
    public boolean isEntryEmpty(){ return this.entry.length() == 0; }
    public boolean writeInAccepted(){ return this.accepted;}
    
    public void deselect(){
	this.accepted = false;
    }
    

    public boolean isNavigateNext(MouseEvent m){    return false;}
    public boolean isNavigatePrev(MouseEvent m){    return false; }
    public boolean isNavigateSummary(MouseEvent m){ return false;}



    private EventMessage entryUpdate( String s){
	if( this.entry.length() < MAX_WRITE_IN_LENGTH)
	    this.entry.append( s );
	this.display.setText(entry.toString() + " ");
	
	return EventMessage.newWriteInMSG( this.entry.toString() );
    }
    
    private void doAccept(){
	if( this.entry.length() > 0 )
	    this.accepted = true;
    }
    
    private void doCancel(){
	this.accepted = false;
    }
    
    private EventMessage doClear(){
	this.entry.delete(0, this.entry.length());
	return this.entryUpdate("");
    }
    
    private EventMessage doBackSpace(){
	if( this.entry.length() > 0)
	    this.entry.delete(this.entry.length() -1 , this.entry.length());
	return this.entryUpdate("");
    }
    
    private EventMessage doKeyPress(MouseEvent m){
	for( FakeButton b : this.charKeys)
	    if( b.isShowing() && b.containsPoint(m))
		return this.entryUpdate(  b.toString() );
	    
	return EventMessage.newMissMSG();
    }
    
    public EventMessage handleMouseClick(MouseEvent m){
	EventMessage em = EventMessage.newWriteInMSG();
	
	if(this.accept.isShowing() && this.accept.containsPoint( m ))
	    //this.doAccept();
	    em = EventMessage.newWriteInMSG("Accept");
	else if ( this.cancel.isShowing() && this.cancel.containsPoint( m))
	    em = EventMessage.newWriteInMSG("Cancel");
	    //this.doCancel();
	else if (this.clear.isShowing() && this.clear.containsPoint( m))
	    em = this.doClear();
	else if (this.backspace.isShowing() && this.backspace.containsPoint( m))
	    em = this.doBackSpace();
	else if (this.space.isShowing() && this.space.containsPoint(m))
	    em = this.entryUpdate(" ");
	else
	    em = this.doKeyPress(m);
	
	return em;
    }
}
