/***************************************************************************
 * XmlBallotParser.java                                                    *
 ***************************************************************************/

/****************************************************************************
 * Copyright (c) 2009 The Univserity of Iowa.                               *
 *                                                                          *
 * Permission is hereby granted, free of charge, to any person obtaining a  *
 * copy of this software and associated documentation files (the            *
 * "Software"), to deal in the Software without restriction, including      *
 * without limitation the rights to use, copy, modify, merge, publish,      *
 * distribute, distribute with modifications, sublicense, and/or sell       *
 * copies of the Software, and to permit persons to whom the Software is    *
 * furnished to do so, subject to the following conditions:                 *
 *                                                                          *
 * The above copyright notice and this permission notice shall be included  *
 * in all copies or substantial portions of the Software.                   *
 *                                                                          *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
 * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
 *                                                                          *
 * Except as contained in this notice, the name(s) of the above copyright   *
 * holders shall not be used in advertising or otherwise to promote the     *
 * sale, use or other dealings in this Software without prior written       *
 * authorization.                                                           *
 ****************************************************************************/

/****************************************************************************
 * Paul Cotton                                                              *
 * The University of Iowa                                                   *
 * Department of Computer Science                                           *
 * April 2009                                                               *
 *                                                                          *
 * Parse a givin ballot definition file. Create a Ballot object from that   *
 * file, and return it to the caller of getBallot().                        *                      
 *****************************************************************************/

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class XmlBallotParser extends DefaultHandler{

  private Log log;
  private BallotFactory ballot;
  
  private Integer contestNum;
  
  private ContestFactory contest;
  private String val;
  private String sectionText;
  
  public XmlBallotParser(Log l){ 
    this.log = l;
  }

  public Ballot getBallot(String f){
    this.ballot = new BallotFactory( this.log );

    this.contestNum = 0;
    this.contest = null;//new Contest(this.model,1);

    this.val = "";
    this.sectionText = "";

    this.parseDocument( f );

    return this.ballot.getBallot();
  }
  
  private void parseDocument(String filename) {
    
    SAXParserFactory spf = SAXParserFactory.newInstance();
    try {
      SAXParser sp = spf.newSAXParser();
      sp.parse(filename, this);
    }
    catch(SAXException se) { se.printStackTrace();}
    catch(ParserConfigurationException pce) {pce.printStackTrace();}
    catch(IOException ie) { ie.printStackTrace();}
  }
  
  
  
  //Event Handlers
  public void startElement(String uri, String localName, String qName,
			   Attributes attributes) throws SAXException {
    this.val = "";//reset

    if(qName.equals("Election")){}
    
    if(qName.equals("Section")){
      this.sectionText = attributes.getValue("value");
      this.ballot.startSection(this.sectionText);
    }	
    
    if(qName.equals("Page"))
      this.ballot.addNewPage();


    if(qName.equals("TextPage"))
      this.ballot.addTextPage(attributes.getValue("text"));

    if(qName.equals("TailPage"))
      this.ballot.addTailPage(attributes.getValue("text"));

    
    if(qName.equals("Contest")){ 
      int maxSel = Integer.parseInt( attributes.getValue("maxSelections"));
      String title = attributes.getValue("title");
      this.contest = new ContestFactory( this.log, maxSel, this.sectionText, title);
      
      try{
	  this.contest.setTwoColumnView( attributes.getValue("twocolumn").equals("true") );}
      catch(NullPointerException e){System.out.println("");}//don't care

      this.contest.setContestNum( contestNum++);
    }
    
    

    if(qName.equals("Race"))
      this.contest.setRaceText(attributes.getValue("value"));

    if(qName.equals("Instructions"))
      this.contest.setInstructionText(attributes.getValue("value"));
    
    if(qName.equals("TwoColumn"))
      this.contest.setTwoColumnView(true);

    if(qName.equals("HideSection"))
      this.contest.hideSectionText(true);

    if(qName.equals("WriteIn")){
      String str = attributes.getValue("value");
      if( str.equals("true") )
        this.contest.enableWriteIn(true);
      else if( str.equals("false"))
        this.contest.enableWriteIn(false);
      else
        System.err.println("WriteIn value must be\"true\" of \"false\"");
    }
    
    if(qName.equals("Candidate"))
      this.contest.addCandidate(attributes.getValue("title"),
                                attributes.getValue("stitle"),
                                attributes.getValue("sstitle"));
  }
  
  
  public void characters(char[] ch, int start, int length) throws SAXException {
    this.val = new String(ch,start,length);
  }
  
  public void endElement(String uri, String localName,
                         String qName) throws SAXException {
    
    if(qName.equals("Election")){ }
    
    if(qName.equals("Section"))
      this.ballot.endSection();
    
    if(qName.equals("Page")) {}
        
    if(qName.equals("Contest")) 
	this.ballot.addContest( this.contest.getContest());
  }  
}

